/**
 * Class        LcdMenu.cpp
 * Author       2021-02-02 Charles Geiser (https://www.dodeka.ch)
 * 
 * Purpose      Implements a menu class which handles a menu displayed on an LCD
 *              The methods are: 
 *                  nextMenuItem()  --> move selection to the next menu item
 *                  prevMenuItem()  --> move the selection to the previous menu item
 *                  show()          --> show the menu on the display  
 *                  doAction()      --> execute the action assigned to the selected menu item
 * 
 *              The selected menu item is markked with a right arrow: ->selected item
 * 
 * Board        LCD with I2C interface    
 * 
 * Remarks      The menu items are automatically moved up or down when selection 
 *              reaches end or top of a page
 * 
 * References    
 */

#include "LcdMenu.h"

/**
 *  Move selection to the next menuitem
 */
void LcdMenu::nextMenuItem()
{
  _previousMenuPage = _menuPage;
  _selectedMenuItem++;
  _selectedMenuItem %= _nbrMenuItems;
  _menuPage = _selectedMenuItem / _nbrDisplayedMenuItems;
  _startMenuItem = _menuPage * _nbrDisplayedMenuItems;
  _stopMenuItem = _startMenuItem + _nbrDisplayedMenuItems;
  if (_stopMenuItem > _nbrMenuItems) { _stopMenuItem = _nbrMenuItems; }

  // If new page, clear the menu and start a new page otherwise just
  // overwrite old entries to prevent flickering.
  show(_previousMenuPage != _menuPage); 
}

/**
 *  Move selection to the previous menuitem
 */
void LcdMenu::prevMenuItem()
{
  _previousMenuPage = _menuPage;
  _selectedMenuItem--;
  _selectedMenuItem = (_selectedMenuItem < 0) ? _selectedMenuItem + _nbrMenuItems : _selectedMenuItem;
  _selectedMenuItem %= _nbrMenuItems;
  _menuPage = _selectedMenuItem / _nbrDisplayedMenuItems;
  _startMenuItem = _menuPage * _nbrDisplayedMenuItems;
  _stopMenuItem = _startMenuItem + _nbrDisplayedMenuItems;
  if (_stopMenuItem > _nbrMenuItems) { _stopMenuItem = _nbrMenuItems; }
  show(_previousMenuPage != _menuPage);
}

/**
 *  Show the menu
 */
void LcdMenu::show(bool clearScreen)
{
  if (clearScreen) _lcd.clear();
  _lcd.setCursor(0,0);

  for(int i = _startMenuItem; i < _stopMenuItem; i++)
  {
    _lcd.setCursor(0, i % _nbrDisplayedMenuItems);
    (i == _selectedMenuItem) ? _lcd.print(LCD_ARROW_R) : _lcd.print(" "); 
    _lcd.print(_menu[i].txt);
  }      
}

/**
 *  Execute the associated menu action
 */
void LcdMenu::doAction()
{
  (_menu[_selectedMenuItem]).action();
}